﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.IO;
using System.Text;
using System.Linq;
using dbfs;
using Xunit;

public class BranchTests
{
	[Fact]
	public void CreateBranch()
	{
		Utils.ClearDB();

		using (DBFS dbfs = Utils.CreateDBFS())
		{
			string name = "test-branch";

			Assert.DoesNotContain(name, dbfs.ListBranches().Select(e => e.Name));

			Assert.NotNull(dbfs.CreateBranch(name, Utils.DEFAULT_USER, Utils.DEFAULT_BRANCH));

			Assert.Contains(name, dbfs.ListBranches().Select(e => e.Name));
		}
	}

	[Fact]
	public void RemoveBranch()
	{
		Utils.ClearDB();

		using (DBFS dbfs = Utils.CreateDBFS())
		{
			string name = "test-branch";

			Assert.DoesNotContain(name, dbfs.ListBranches().Select(e => e.Name));

			Assert.NotNull(dbfs.CreateBranch(name, Utils.DEFAULT_USER, Utils.DEFAULT_BRANCH));

			Assert.Contains(name, dbfs.ListBranches().Select(e => e.Name));

			dbfs.RemoveBranch(name);

			Assert.DoesNotContain(name, dbfs.ListBranches().Select(e => e.Name));
		}
	}
}
