﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using dbfs;
using dbfs.utils;
using Xunit;

public class FileTests
{
	void createFile(DBFS dbfs, string path, string content)
	{
		using (var commit = dbfs.StartCommit("", Utils.DEFAULT_USER, Utils.DEFAULT_BRANCH)) {
			commit.AddFile(path, p => new MemoryStream(Encoding.UTF8.GetBytes(content)));

			var result = commit.Finish();

			Assert.NotNull(result);
			Assert.Contains(path, result.Files.Select(e => e.Path));
		}

		{
			var list = dbfs.GetFilesList(Utils.DEFAULT_BRANCH, new FileUrl(path).Directory)
				.Select(e => e.Path);

			Assert.Contains(path, list);
		}
	}

	void removeFile(DBFS dbfs, string path)
	{
		using (var commit = dbfs.StartCommit("", Utils.DEFAULT_USER, Utils.DEFAULT_BRANCH)) {
			commit.RemoveFile(path);

			var result = commit.Finish();

			Assert.NotNull(result);
			Assert.Equal(1, result.Changes.Count);
			Assert.Contains(path, result.Files.Select(e => e.Path));
		}

		{
			var list = dbfs.GetFilesList(Utils.DEFAULT_BRANCH, new FileUrl(path).Directory)
				.Select(e => e.Path);

			Assert.DoesNotContain(path, list);
		}
	}

	[Fact]
	public void RemoveFile()
	{
		Utils.ClearDB();

		using (DBFS dbfs = Utils.CreateDBFS())
		{
			string path = "/test_file.jpg";

			createFile(dbfs, path, "this is content of a file");
			removeFile(dbfs, path);
		}
	}

	[Fact]
	public void RecreationOfAFile_Issue179()
	{
		Utils.ClearDB();

		using (DBFS dbfs = Utils.CreateDBFS())
		{
			string path = "/test_file.jpg";

			createFile(dbfs, path, "first content");
			removeFile(dbfs, path);

			createFile(dbfs, path, "second");
			removeFile(dbfs, path);
		}
	}
}
