﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using dbfs;
using System;
using System.Diagnostics;
using dbfs.utils;

class Utils
{
	public const string DEFAULT_BRANCH = "Start";
	public const string DEFAULT_USER = "admin";

	public const string CONFIG_NAME = "dbfs-tests";

	static Utils()
	{
		Config config = Config.Get(CONFIG_NAME, true);

		_schemaFolder = config.GetText("tests/schema-folder");
		_clearCommand = config.GetText("tests/db-clear-command");
	}

	public static DBFS CreateDBFS()
	{
		return new DBFS(CONFIG_NAME);
	}

	public static void ClearDB()
	{
		Process process = new Process();

		process.StartInfo.FileName = "cmd";
		process.StartInfo.Arguments = "/c " + _clearCommand;
		process.StartInfo.WorkingDirectory = _schemaFolder;
		process.StartInfo.UseShellExecute = false;
		process.StartInfo.RedirectStandardOutput = true;
		process.StartInfo.RedirectStandardError = true;

		process.Start();
		process.WaitForExit();

		string error = process.StandardError.ReadToEnd();
		if (!String.IsNullOrEmpty(error)) throw new Exception(error);
	}

	static readonly string _schemaFolder;
	static readonly string _clearCommand;
}
