﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs
{
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using dbfs.repository;

	public class Commit
	{
		public int Id { get; private set; }
		public string Message { get; private set; }
		public DateTime Date { get; private set; }
		public string UserName { get; private set; }

		public Branch Branch { get; private set; }

		public List<BaseFileEntity> Changes { get; private set; }
		public List<File> Files { get; private set; }
		public List<Folder> Folders { get; private set; }

		internal Commit(model.Commit commit, Branch branch)
		{
			Id = commit.id;
			Message = commit.message;
			Date = commit.date;
			UserName = commit.User.UserName;

			Branch = branch;

			Changes = commit.FileRevisions.Select(BaseFileEntityFactory.Create).ToList();
			Files = Changes.OfType<File>().ToList();
			Folders = Changes.OfType<Folder>().ToList();
		}

		internal Commit(model.Commit commit, model.Branch branch) : this(commit, new Branch(branch))
		{
		}
	}
}
