﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs
{
	using System;
	using dbfs.model;

	public class File : BaseFileEntity
	{
		public override bool IsDirectory { get { return false; } }

		public string Extension 
		{ 
			get 
			{ 
				int j = Path.LastIndexOf('/');
				int k = Path.LastIndexOf('.');
				return k > j ? Path.Substring(k+1) : "";
			} 
		}

		internal int BlobID { get; private set; }
		public long Size { get; private set; }
		public string Hash { get; private set; }

		internal File(FileRevision revision)
		{
			if (revision.File.is_directory) throw new ArgumentException();

			Id = revision.file_id;
			RevisionID = revision.id;
			Path = revision.File.path;
			Date = revision.Commit.date;
			CommitId = revision.commit_id;
			UserName = revision.Commit.User.UserName;
			ChangeType = revision.Type;

			if (revision.Blob != null) {
				BlobID = revision.blob_id.Value;
				Size = revision.Blob.size.Value;
				Hash = revision.Blob.sha1;
			}
		}
	}
}
