﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace dbfs.utils
{
	public static class DictionaryUtils
	{
		/// <summary>
		/// Inserts range into dictionary. Overwrites all duplicates.
		/// </summary>
		/// <returns>dictionary itself</returns>
		public static IDictionary<TKey, TValue> AddRange<TKey, TValue>(this IDictionary<TKey, TValue> target,
			IEnumerable<TValue> range,
			Func<TValue, TKey> key_selector)
		{
			target.ThrowIfNull("target");

			foreach (var value in range) {
				target[key_selector(value)] = value;
			}

			return target;
		}

		/// <summary>
		/// Inserts range into dictionary. Overwrites all duplicates.
		/// </summary>
		/// <returns>dictionary itself</returns>
		public static IDictionary<TKey, TValue> AddRange<TKey, TValue, TAny>(this IDictionary<TKey, TValue> target,
			IEnumerable<TAny> range,
			Func<TAny, TKey> key_selector,
			Func<TAny, TValue> value_selector)
		{
			target.ThrowIfNull("target");

			foreach (var e in range) {
				target[key_selector(e)] = value_selector(e);
			}

			return target;
		}

		/// <summary>
		/// Removes all elements that match the conditions defined by the specified predicate.
		/// </summary>
		/// <returns>The number of elements that were removed.</returns>
		public static int RemoveWhere<T, P>(this IDictionary<T, P> target, Predicate<T> match)
		{
			target.ThrowIfNull("target");

			int result = 0;

			foreach (var key in target.Keys.ToList()) {
				if (match(key)) {
					target.Remove(key);
					++result;
				}
			}

			return result;
		}
	}
}
