﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;

namespace dbfs.utils.db
{
	public class FieldReaderStream : Stream
	{
		public FieldReaderStream(IBinaryFieldReader reader)
		{
			_reader = reader;
		}

		public override bool CanRead { get { return true; } }
		public override bool CanWrite { get { return false; } }

		public override int Read(byte[] buffer, int offset, int count)
		{
			return _reader.Read(buffer, offset, count);
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			throw new NotSupportedException();
		}

		public override bool CanSeek { get { return true; } }
		public override void Flush() { throw new NotSupportedException(); }
		public override long Length { get { return _reader.Size; } }
		public override void SetLength(long value) { throw new NotSupportedException(); }

		public override long Seek(long offset, SeekOrigin origin)
		{
			return Position =
				origin == SeekOrigin.Begin ? offset :
				origin == SeekOrigin.Current ? Position + offset :
				Length + offset;
		}

		public override long Position
		{
			get { return _reader.Position; }
			set { _reader.Position = (int)value; }
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (_reader != null)
				{
					_reader.Dispose();
					_reader = null;
				}
			}

			base.Dispose(disposing);
		}

		private IBinaryFieldReader _reader;
	}
}
