﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Data;
using System.Data.Common;
using System.Linq;

namespace dbfs.utils.db.impl
{
	class BinaryFieldWriter : IBinaryFieldWriter
	{
		public BinaryFieldWriter(DbConnection connection, string schema, string table, string write_field, string key_field, object key)
		{
			_cmd = connection.CreateCommand();
			_cmd.CommandTimeout = 5 * 60;

			_cmd.CommandType = CommandType.Text;
			_cmd.CommandText = string.Format(
				"UPDATE {0} SET [{2}] = @Content WHERE [{1}] = @KeyValue",
				Helpers.TableSource(schema, table), write_field, key_field
			);

			DbParameter keyValue = _cmd.CreateParameter();
			keyValue.ParameterName = "@KeyValue";
			keyValue.Value = key;
			_cmd.Parameters.Add(keyValue);

			_content = _cmd.CreateParameter();
			_content.ParameterName = "@Content";
			_content.DbType = DbType.Binary;
			_cmd.Parameters.Add(_content);
		}

		public int Position
		{
			get { throw new NotSupportedException(); }
			set { throw new NotSupportedException(); }
		}

		public virtual void Write(byte[] buffer, int offset, int count)
		{
			if (_cmd == null)
				throw new InvalidOperationException("Sequential write operations are not supported");

			buffer = buffer.Skip(offset).Take(count).ToArray();

			_content.Value = buffer;
			_content.Size = buffer.Length;

			_cmd.ExecuteNonQuery();

			Dispose();
		}

		public virtual void Dispose()
		{
			if (_cmd != null)
			{
				_cmd.Dispose();
				_cmd = null;
			}
		}

		private DbCommand _cmd;
		private DbParameter _content;
	}
}
