﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.Data;
using System.Data.Common;
using System.Linq;

namespace dbfs.utils.db.impl.mssql
{
	class MsSqlImageFieldWriter : IBinaryFieldWriter
	{
		public MsSqlImageFieldWriter(DbConnection connection, string table, string write_field, string key_field, object key)
		{
			using (DbCommand cmd = connection.CreateCommand())
			{
				cmd.CommandType = CommandType.Text;
				cmd.CommandTimeout = 5 * 60;
				cmd.CommandText = string.Format(
					"UPDATE [{1}] SET [{0}]=0x0 WHERE [{2}]=@KeyValue; SELECT @Pointer=TEXTPTR([{0}]) FROM [{1}] WHERE [{2}]=@KeyValue",
					write_field, table, key_field
				);

				DbParameter keyValue = cmd.CreateParameter();
				keyValue.ParameterName = "@KeyValue";
				keyValue.Value = key;

				cmd.Parameters.Add(keyValue);

				DbParameter pointer = cmd.CreateParameter();
				pointer.ParameterName = "@Pointer";
				pointer.DbType = DbType.Binary;
				pointer.Size = 16;
				pointer.Direction = ParameterDirection.Output;

				cmd.Parameters.Add(pointer);

				cmd.ExecuteNonQuery();

				_pointer = (byte[])pointer.Value;
			}

			_cmd = connection.CreateCommand();
			_cmd.CommandTimeout = 5 * 60;

			_cmd.CommandType = CommandType.Text;
			_cmd.CommandText = string.Format(
				"UPDATETEXT [{1}].[{0}] @Pointer @Offset @Delete @Bytes",
				write_field, table
			);

			_pointerParam = _cmd.CreateParameter();
			_pointerParam.ParameterName = "@Pointer";
			_pointerParam.DbType = DbType.Binary;
			_pointerParam.Size = 16;
			_cmd.Parameters.Add(_pointerParam);

			_offsetParam = _cmd.CreateParameter();
			_offsetParam.ParameterName = "@Offset";
			_offsetParam.DbType = DbType.Int32; //SqlDbType.Int
			_cmd.Parameters.Add(_offsetParam);

			_deleteParam = _cmd.CreateParameter();
			_deleteParam.ParameterName = "@Delete";
			_deleteParam.DbType = DbType.Int32; //SqlDbType.Int
			_deleteParam.Value = (int)1; // erase special 1st byte we set as initial value to get pointer
			_cmd.Parameters.Add(_deleteParam);

			_bytesParam = _cmd.CreateParameter();
			_bytesParam.ParameterName = "@Bytes";
			_bytesParam.DbType = DbType.Binary;
			_cmd.Parameters.Add(_bytesParam);

			Write(new byte[0], 0, 0);
		}

		public int Position { get; set; }

		public void Write(byte[] buffer, int offset, int count)
		{
			buffer = buffer.Skip(offset).Take(count).ToArray();

			_pointerParam.Value = _pointer;
			_offsetParam.Value = Position;
			_bytesParam.Value = buffer;
			_bytesParam.Size = buffer.Length;

			_cmd.ExecuteNonQuery();

			_deleteParam.Value = (int)0;
			Position += buffer.Length;
		}

		public void Dispose()
		{
			if (_cmd != null)
			{
				_pointerParam = null;
				_offsetParam = null;
				_deleteParam = null;
				_bytesParam = null;

				_cmd.Dispose();
				_cmd = null;
				_pointer = null;
			}
		}

		private byte[] _pointer;
		private DbCommand _cmd;
		private DbParameter _pointerParam;
		private DbParameter _offsetParam;
		private DbParameter _deleteParam;
		private DbParameter _bytesParam;
	}
}
